@@ -262,6 +262,12 @@ void RedisConnection::Connect(asio::yield_context& yc)
 
 	boost::asio::deadline_timer timer (m_Strand.context());
 
+	auto waitForReadLoop ([this, &yc]() {
+		while (!m_Queues.FutureResponseActions.empty()) {
+			IoEngine::YieldCurrentCoroutine(yc);
+		}
+	});
+
 	for (;;) {
 		try {
 			if (m_Path.IsEmpty()) {
@@ -294,6 +300,7 @@ void RedisConnection::Connect(asio::yield_context& yc)
 					}
 
 					Handshake(conn, yc);
+					waitForReadLoop();
 					m_TlsConn = std::move(conn);
 				} else {
 					Log(m_Parent ? LogNotice : LogInformation, "IcingaDB")
@@ -305,6 +312,7 @@ void RedisConnection::Connect(asio::yield_context& yc)
 
 					icinga::Connect(conn->next_layer(), m_Host, Convert::ToString(m_Port), yc);
 					Handshake(conn, yc);
+					waitForReadLoop();
 					m_TcpConn = std::move(conn);
 				}
 			} else {
@@ -317,6 +325,7 @@ void RedisConnection::Connect(asio::yield_context& yc)
 
 				conn->next_layer().async_connect(Unix::endpoint(m_Path.CStr()), yc);
 				Handshake(conn, yc);
+				waitForReadLoop();
 				m_UnixConn = std::move(conn);
 			}
 
@@ -413,12 +422,16 @@ void RedisConnection::ReadLoop(asio::yield_context& yc)
 								throw;
 							} catch (...) {
 								promise.set_exception(std::current_exception());
-
-								continue;
+								break;
 							}
 						}
 
-						promise.set_value(std::move(replies));
+						try {
+							promise.set_value(std::move(replies));
+						} catch (const std::future_error&) {
+							// Complaint about the above op is not allowed
+							// due to promise.set_exception() was already called
+						}
 					}
 			}
 		}
